from Components.FileList import FileList, EXTENSIONS
from Components.Harddisk import harddiskmanager
from Components.ParentalControl import parentalControl, IMG_WHITESERVICE, \
	IMG_WHITEBOUQUET, IMG_BLACKSERVICE, IMG_BLACKBOUQUET
from MenuList import MenuList
from Tools.Directories import SCOPE_SKIN_IMAGE, SCOPE_CURRENT_SKIN, resolveFilename, fileExists
from Tools.LoadPixmap import LoadPixmap
from enigma import RT_HALIGN_LEFT, eListboxPythonMultiContent, eServiceReference, \
	eServiceCenter, gFont
from os import path as os_path, listdir
from re import compile as re_compile

REF_DIR = "2:0:1:0:0:0:0:0:0:0:"

#Now there is a list of pictures instead of one...
entryPicture = {}

entryPicture[IMG_BLACKSERVICE] = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/lock.png"))
entryPicture[IMG_BLACKBOUQUET] = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/lockBouquet.png"))
entryPicture[IMG_WHITESERVICE] = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/unlock.png"))
entryPicture[IMG_WHITEBOUQUET] = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/unlockBouquet.png"))

def ParentalControlEntryComponent(service, name):
	protectionType = parentalControl.getProtectionType(service)
	locked = protectionType[0]
	sImage = protectionType[1]
	res = [
		(service, name, locked),
		(eListboxPythonMultiContent.TYPE_TEXT, 80, 5, 300, 50, 0, RT_HALIGN_LEFT, name)
	]
	#Changed logic: The image is defined by sImage, not by locked anymore
	if sImage != "":
		res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 0, 0, 32, 32, entryPicture[sImage]))
	return res

class ParentalControlList(MenuList):

	def __init__(self, list, enableWrapAround = False):
		MenuList.__init__(self, list, enableWrapAround, eListboxPythonMultiContent)
		self.l.setFont(0, gFont("Regular", 20))
		self.l.setItemHeight(32)

	def toggleSelectedLock(self, protect):
		curSel = self.l.getCurrentSelection()
		if protect == None:
			protect = not curSel[0][2]		
		
		if protect:
			parentalControl.protectService(curSel[0][0])	
		else:
			parentalControl.unProtectService(curSel[0][0])
		#Instead of just negating the locked- flag, now I rebuild the current entry every time
		self.list[self.l.getCurrentSelectionIndex()] = ParentalControlEntryComponent(curSel[0][0], curSel[0][1])
		self.l.setList(self.list)

def ParentalControlFileEntryComponent(name, absolute = None, isDir = False, canBeProtected = False):
	
	if canBeProtected:
		if isinstance(absolute, eServiceReference):
			service = absolute.toCompareString()
		else:
			service = REF_DIR + absolute
		protectionType = parentalControl.getProtectionType(service)
		locked = protectionType[0]
		sImage = protectionType[1]	
	else:
		service = ""
		sImage = ""
		locked = False
		
	res = [ (absolute, isDir, service, name, locked, canBeProtected) ]
	res.append((eListboxPythonMultiContent.TYPE_TEXT, 80, 5, 300, 50, 0, RT_HALIGN_LEFT, name))
	if isDir:
		png = LoadPixmap(cached=True, path=resolveFilename(SCOPE_CURRENT_SKIN, "extensions/directory.png"))
	else:
		extension = name.split('.')
		extension = extension[-1].lower()
		if EXTENSIONS.has_key(extension):
			png = LoadPixmap(resolveFilename(SCOPE_CURRENT_SKIN, "extensions/" + EXTENSIONS[extension] + ".png"))
		else:
			png = None
	if png is not None:
		res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 55, 2, 20, 20, png))
	
	if sImage != "":
		res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 0, 0, 32, 32, entryPicture[sImage]))

	return res

class ParentalControlFileList(FileList):
	def __init__(self, directory, showDirectories = True, showFiles = True, showMountpoints = True, matchingPattern = None, inhibitDirs = False, inhibitMounts = False, isTop = False, enableWrapAround = True, additionalExtensions = None):
		useServiceRef = True
		FileList.__init__(self, directory, showDirectories, showFiles, showMountpoints, matchingPattern, useServiceRef, inhibitDirs, inhibitMounts, isTop, enableWrapAround, additionalExtensions)

	def toggleSelectedLock(self, protect):
		curSel = self.l.getCurrentSelection()
		absolute = curSel[0][0] 
		isDir = curSel[0][1] 
		service = curSel[0][2] 
		name = curSel[0][3] 
		locked = curSel[0][4] 
		canBeProtected = curSel[0][5] 
		if canBeProtected:
			if protect == None:
				protect = not locked	
			if protect:
				parentalControl.protectService(service)	
			else:
				parentalControl.unProtectService(service)
			#Instead of just negating the locked- flag, now I rebuild the current entry every time
			self.list[self.l.getCurrentSelectionIndex()] = ParentalControlFileEntryComponent(name, absolute, isDir, canBeProtected)
			self.l.setList(self.list)
	
	def changeDir(self, directory, select = None):
		self.list = []

		# if we are just entering from the list of mount points:
		if self.current_directory is None:
			if directory and self.showMountpoints:
				self.current_mountpoint = self.getMountpointLink(directory)
			else:
				self.current_mountpoint = None
		self.current_directory = directory
		directories = []
		files = []

		if directory is None and self.showMountpoints: # present available mountpoints
			for p in harddiskmanager.getMountedPartitions():
				path = os_path.join(p.mountpoint, "")
				if path not in self.inhibitMounts and not self.inParentDirs(path, self.inhibitDirs):
					self.list.append(ParentalControlFileEntryComponent(name = p.description, absolute = path, isDir = True, canBeProtected = False))
			files = [ ]
			directories = [ ]
		elif directory is None:
			files = [ ]
			directories = [ ]
		elif self.useServiceRef:
			root = eServiceReference(REF_DIR + directory)
			if self.additional_extensions:
				root.setName(self.additional_extensions)
			serviceHandler = eServiceCenter.getInstance()
			list = serviceHandler.list(root)

			while 1:
				s = list.getNext()
				if not s.valid():
					del list
					break
				if s.flags & s.mustDescent:
					directories.append(s.getPath())
				else:
					files.append(s)
			directories.sort()
			files.sort()
		else:
			if fileExists(directory):
				try:
					files = listdir(directory)
				except:
					files = []
				files.sort()
				tmpfiles = files[:]
				for x in tmpfiles:
					if os_path.isdir(directory + x):
						directories.append(directory + x + "/")
						files.remove(x)

		if directory is not None and self.showDirectories and not self.isTop:
			if directory == self.current_mountpoint and self.showMountpoints:
				self.list.append(ParentalControlFileEntryComponent(name = "<" +_("List of Storage Devices") + ">", absolute = None, isDir = True, canBeProtected = False))
			elif (directory != "/") and not (self.inhibitMounts and self.getMountpoint(directory) in self.inhibitMounts):
				self.list.append(ParentalControlFileEntryComponent(name = "<" +_("Parent Directory") + ">", absolute = '/'.join(directory.split('/')[:-2]) + '/', isDir = True, canBeProtected = False))

		if self.showDirectories:
			for x in directories:
				if not (self.inhibitMounts and self.getMountpoint(x) in self.inhibitMounts) and not self.inParentDirs(x, self.inhibitDirs):
					name = x.split('/')[-2]
					self.list.append(ParentalControlFileEntryComponent(name = name, absolute = x, isDir = True, canBeProtected = True))

		if self.showFiles:
			for x in files:
				if self.useServiceRef:
					path = x.getPath()
					name = path.split('/')[-1]
				else:
					path = directory + x
					name = x

				if (self.matchingPattern is None) or re_compile(self.matchingPattern).search(path):
					self.list.append(ParentalControlFileEntryComponent(name = name, absolute = x , isDir = False, canBeProtected = True))

		self.l.setList(self.list)

		if select is not None:
			i = 0
			self.moveToIndex(0)
			for x in self.list:
				p = x[0][0]
				
				if isinstance(p, eServiceReference):
					p = p.getPath()
				
				if p == select:
					self.moveToIndex(i)
				i += 1
